@@ -8,7 +8,14 @@ from pathlib import Path
 import click
 
 from . import __version__, __repo__
-from .base import SHUTIL_ARCHIVE_FORMATS, SHUTIL_ARCHIVE_SUFFIX_MAP
+from .base import (
+    SHUTIL_ARCHIVE_FORMATS,
+    SHUTIL_ARCHIVE_SUFFIX_MAP,
+    AddCommand,
+    RemoveCommand,
+    LinkMode,
+    ExportMode,
+)
 from .control import CommandRunner
 from .filesystem import (
     ProjectPath,
@@ -41,19 +48,17 @@ from .template import (
 )
 
 if TYPE_CHECKING:
-    from .base import LinkMode, NAMES, RepoVisibility
+    from .base import NAMES, RepoVisibility
     from typing import Optional, Iterable, List, Literal, Dict, Callable
     from .control import AtomicIterable
 
 
 def process_atoms(load_template: Optional[bool] = True):
-    # *validation_funcs: Callable[[ProjectPath], bool], pass_template_name=False
     """Custom decorator which passes the object after performing some state verification on it."""
 
     def state_constructor(template: Optional[str] = None) -> Callable[[], Dict]:
         def state_init() -> Dict:
             dct = {
-                "linked": {NAMES.convert_mode(mode): [] for mode in NAMES.modes},
                 "template_modifications": [],
             }
             names = {"template": template} if template is not None else {}
@@ -62,22 +67,24 @@ def process_atoms(load_template: Optional[bool] = True):
         return state_init
 
     def decorator(f):
-        if load_template is None:
+        def _helper(ctx, template, *args, **kwargs):
+            if load_template is None:
+                template_dict = None
+            elif load_template:
+                template_dict = toml_load_local_template(ctx.obj["proj_path"].template)
+            else:
+                template_dict = template_linker.load_template(template)
 
-            @click.pass_context
-            def new_func_0(ctx, *args, **kwargs):
-                command_iter = ctx.invoke(f, *args, **kwargs)
-                runner = CommandRunner(
-                    ctx.obj["proj_path"],
-                    None,
-                    dry_run=ctx.obj["dry_run"],
-                    verbose=ctx.obj["verbose"],
-                )
-                runner.execute(command_iter, state_init=state_constructor())
+            CommandRunner(
+                ctx.obj["proj_path"],
+                template_dict,
+                dry_run=ctx.obj["dry_run"],
+                verbose=ctx.obj["verbose"],
+            ).execute(
+                ctx.invoke(f, *args, **kwargs), state_init=state_constructor(template)
+            )
 
-            return update_wrapper(new_func_0, f)
-
-        elif not load_template:
+        if load_template is False:
 
             @click.argument(
                 "template",
@@ -85,39 +92,22 @@ def process_atoms(load_template: Optional[bool] = True):
                 metavar="TEMPLATE",
             )
             @click.pass_context
-            def new_func_1(ctx, template: str, *args, **kwargs):
-                command_iter = ctx.invoke(f, *args, **kwargs)
-                runner = CommandRunner(
-                    ctx.obj["proj_path"],
-                    template_linker.load_template(template),
-                    dry_run=ctx.obj["dry_run"],
-                    verbose=ctx.obj["verbose"],
-                )
-                runner.execute(command_iter, state_init=state_constructor(template))
+            def wrapper_with_template(ctx, template, *args, **kwargs):
+                _helper(ctx, template, *args, **kwargs)
 
-            return update_wrapper(new_func_1, f)
+            return update_wrapper(wrapper_with_template, f)
 
         else:
 
             @click.pass_context
-            def new_func(ctx, *args, **kwargs):
-                command_iter = ctx.invoke(f, *args, **kwargs)
-                runner = CommandRunner(
-                    ctx.obj["proj_path"],
-                    toml_load_local_template(ctx.obj["proj_path"].template),
-                    dry_run=ctx.obj["dry_run"],
-                    verbose=ctx.obj["verbose"],
-                )
-                runner.execute(command_iter, state_init=state_constructor())
+            def wrapper(ctx, *args, **kwargs):
+                _helper(ctx, None, *args, **kwargs)
 
-            return update_wrapper(new_func, f)
+            return update_wrapper(wrapper, f)
 
     return decorator
 
 
-# TODO
-# figure out how to just stick these wrappers directly inside the process_atoms function
-# then -n and --verbose can be specified at the end, rather than right at the front
 @click.group()
 @click.version_option(prog_name="tpr (texproject)")
 @click.option(
@@ -151,7 +141,7 @@ def cli(ctx, proj_dir: Path, dry_run: bool, verbose: bool) -> None:
     }
 
 
-@cli.command()
+@cli.command(short_help="Initialize a new project.")
 @process_atoms(load_template=False)
 def init() -> Iterable[AtomicIterable]:
     """Initialize a new project in the working directory. The project is created using
@@ -161,7 +151,7 @@ def init() -> Iterable[AtomicIterable]:
     intermediate directories are automatically constructed.
     """
     # must link templates before writing
-    yield OutputFolderCreator()
+    yield OutputFolderCreator.with_abort()
     yield TemplateDictLinker()
     yield InfoFileWriter()
 
@@ -179,18 +169,19 @@ def init() -> Iterable[AtomicIterable]:
 )
 @process_atoms(load_template=None)
 def config(config_file: Literal["local", "global"]) -> Iterable[AtomicIterable]:
-    """Edit texproject configuration files. This opens the corresponding file in your
-    $EDITOR. By default, edit the project template file: this requires the working
+    """Edit configuration files. This opens the corresponding file in your
+    $EDITOR. By default, edit the local template file: this requires the working
     directory to be texproject directory.
-
-    Note that this command does not replace existing macro files. See the `tpr import`
-    command for this functionality.
     """
     yield FileEditor(config_file)
 
 
 def _link_option(mode: LinkMode):
-    linker = {"macro": macro_linker, "citation": citation_linker, "style": style_linker}
+    linker = {
+        LinkMode.macro: macro_linker,
+        LinkMode.citation: citation_linker,
+        LinkMode.style: style_linker,
+    }
     return click.option(
         f"--{mode}",
         f"{NAMES.convert_mode(mode)}",
@@ -214,12 +205,12 @@ def _path_option(mode: LinkMode):
 
 
 @cli.command("import")
-@_link_option("macro")
-@_link_option("citation")
-@_link_option("style")
-@_path_option("macro")
-@_path_option("citation")
-@_path_option("style")
+@_link_option(LinkMode.macro)
+@_link_option(LinkMode.citation)
+@_link_option(LinkMode.style)
+@_path_option(LinkMode.macro)
+@_path_option(LinkMode.citation)
+@_path_option(LinkMode.style)
 @click.option(
     "--gitignore",
     "gitignore",
@@ -291,7 +282,7 @@ def validate(pdf: Optional[Path], logfile: Optional[Path]) -> Iterable[AtomicIte
     )
 
 
-@cli.command()
+@cli.command(short_help="Create compressed exports.")
 @click.option(
     "--format",
     "compression",
@@ -301,7 +292,7 @@ def validate(pdf: Optional[Path], logfile: Optional[Path]) -> Iterable[AtomicIte
 @click.option(
     "--mode",
     "mode",
-    type=click.Choice(["arxiv", "build", "source"]),
+    type=click.Choice(ExportMode),  # type: ignore
     default="source",
     show_default=True,
     help="specify what to export",
@@ -309,7 +300,7 @@ def validate(pdf: Optional[Path], logfile: Optional[Path]) -> Iterable[AtomicIte
 @click.argument("output", type=click.Path(exists=False, writable=True, path_type=Path))
 @process_atoms()
 def archive(
-    compression: str, mode: Literal["archive", "build", "source"], output: Path
+    compression: str, mode: ExportMode, output: Path
 ) -> Iterable[AtomicIterable]:
     """Create a compressed export with name OUTPUT. If the 'arxiv' or 'build' options are
     chosen, 'latexmk' is used to compile additional required files.
@@ -335,7 +326,8 @@ def archive(
      xztar: xz'ed tar-file
      zip: ZIP file
 
-    Note that not all compression modes may be available on your system.
+    Note that some compression modes may not be available on your system. The available
+    options are listed below.
     """
     if compression is None:
         try:
@@ -353,9 +345,9 @@ def template() -> None:
 
 
 @template.command()
-@_link_option("macro")
-@_link_option("citation")
-@_link_option("style")
+@_link_option(LinkMode.macro)
+@_link_option(LinkMode.citation)
+@_link_option(LinkMode.style)
 @click.option("--index", "index", help="position to insert", default=0, type=int)
 @process_atoms()
 def add(
@@ -367,30 +359,37 @@ def add(
     """Add entries to the template dictionary. The --index option allows you to specify
     the index to insert the citation (--index 0 means to insert at the beginning).
     """
-    for mode, names in [("macro", macros), ("citation", citations), ("style", styles)]:
-        yield ApplyModificationSequence((mode, "add", name, index) for name in names)
-
+    for mode, names in [
+        (LinkMode.macro, macros),
+        (LinkMode.citation, citations),
+        (LinkMode.style, styles),
+    ]:
+        yield ApplyModificationSequence(AddCommand(mode, name, index) for name in names)
     yield TemplateDictWriter()
 
 
 @template.command()
-@_link_option("macro")
-@_link_option("citation")
-@_link_option("style")
+@_link_option(LinkMode.macro)
+@_link_option(LinkMode.citation)
+@_link_option(LinkMode.style)
 @process_atoms()
 def remove(
     macros: List[str], citations: List[str], styles: List[str]
 ) -> Iterable[AtomicIterable]:
     """Remove entries from the template dictionary."""
-    for mode, names in [("macro", macros), ("citation", citations), ("style", styles)]:
-        yield ApplyModificationSequence((mode, "remove", name) for name in names)
-
+    for mode, names in [
+        (LinkMode.macro, macros),
+        (LinkMode.citation, citations),
+        (LinkMode.style, styles),
+    ]:
+        yield ApplyModificationSequence(RemoveCommand(mode, name) for name in names)
     yield TemplateDictWriter()
 
 
 @template.command()
 @process_atoms()
 def edit():
+    """Open the template dictionary in your $EDITOR."""
     yield FileEditor("template")
 
 
@@ -463,7 +462,7 @@ def git_init(
     """
     yield GitFileWriter()
     yield InitializeGitRepo()
-    yield CreateGithubRepo(repo_name, repo_desc, vis, wiki, issues)
+    yield CreateGithubRepo.with_abort(repo_name, repo_desc, vis, wiki, issues)
     yield WriteGithubApiToken(repo_name)
 
 
@@ -487,8 +486,8 @@ def init_files(force: bool) -> Iterable[AtomicIterable]:
 )
 @process_atoms()
 def init_archive(repo_name: str) -> Iterable[AtomicIterable]:
-    """Set the GitHub secret and archive repository."""
-    yield LatexBuildWriter(force=True)
+    """Set the GitHub secret and archive repository. Run tpr git init --help for more information."""
+    yield LatexBuildWriter.with_abort(force=True)
     yield WriteGithubApiToken(repo_name)
 
 
@@ -500,7 +499,7 @@ def util() -> None:
 @util.command("upgrade")
 @process_atoms(load_template=None)
 def upgrade() -> Iterable[AtomicIterable]:
-    """Upgrade project data structure from previous versions."""
+    """Upgrade the project data structure from previous versions."""
     yield UpgradeProject()
 
 
@@ -525,9 +524,9 @@ def clean() -> Iterable[AtomicIterable]:
 
 
 @util.command()
-@_link_option("macro")
-@_link_option("citation")
-@_link_option("style")
+@_link_option(LinkMode.macro)
+@_link_option(LinkMode.citation)
+@_link_option(LinkMode.style)
 @process_atoms()
 def diff(
     macros: List[str],
@@ -548,13 +547,15 @@ def show_config():
 
 
 @cli.command("list")
-@click.argument("res_class", type=click.Choice(NAMES.modes + ("template",)))
-def list_(res_class: LinkMode | Literal["template"]) -> None:
+@click.argument(
+    "res_class", type=click.Choice([e.value for e in LinkMode] + ["template"])
+)
+def list_(res_class: Literal["macro", "citation", "style", "template"]) -> None:
     """Retrieve program and template information."""
     linker_map = {
-        "citation": citation_linker,
-        "macro": macro_linker,
-        "style": style_linker,
+        LinkMode.citation.value: citation_linker,
+        LinkMode.macro.value: macro_linker,
+        LinkMode.style.value: style_linker,
         "template": template_linker,
     }
 
