@@ -1,7 +1,9 @@
 from __future__ import annotations
-import os
 from typing import TYPE_CHECKING
 
+from dataclasses import dataclass
+import os
+
 import keyring
 
 from .control import AtomicIterable, RuntimeClosure, SUCCESS, FAIL
@@ -38,17 +40,7 @@ def git_has_remote(path: Path):
     )
 
 
-def github_remote_repo(path: Path) -> str:
-    # todo: think about formatting here
-    proc = subprocess.run(
-        ["git", "config", "--get", "remote.origin.url"], cwd=path, capture_output=True
-    )
-    if proc.returncode == 0:
-        return proc.stdout.decode("ascii").split(":")[1]
-    else:
-        raise Exception("No remote!")
-
-
+@dataclass
 class InitializeGitRepo(AtomicIterable):
     def __call__(
         self, proj_path: ProjectPath, template_dict: Dict, state: Dict, temp_dir: Path
@@ -66,20 +58,13 @@ class InitializeGitRepo(AtomicIterable):
                 run_command(proj_path, command)
 
 
+@dataclass
 class CreateGithubRepo(AtomicIterable):
-    def __init__(
-        self,
-        repo_name: str,
-        description: str,
-        visibility: RepoVisibility,
-        wiki: bool,
-        issues: bool,
-    ) -> None:
-        self._repo_name = repo_name
-        self._description = description
-        self._visibility = visibility
-        self._wiki = wiki
-        self._issues = issues
+    repo_name: str
+    description: str
+    visibility: RepoVisibility
+    wiki: bool
+    issues: bool
 
     def __call__(
         self, proj_path: ProjectPath, template_dict: Dict, state: Dict, temp_dir: Path
@@ -91,36 +76,36 @@ class CreateGithubRepo(AtomicIterable):
             )
         else:
             if org is not None:
-                repo_name = f"{org}/{self._repo_name}"
+                repo_name = f"{org}/{self.repo_name}"
             else:
-                repo_name = self._repo_name
+                repo_name = self.repo_name
             gh_command = [
                 "gh",
                 "repo",
                 "create",
                 "-d",
-                self._description,
+                self.description,
                 "--source",
                 str(proj_path.dir),
                 "--remote",
                 "origin",
                 "--push",
                 repo_name,
-                "--" + self._visibility,
+                "--" + self.visibility,
             ]
 
-            if not self._wiki:
+            if not self.wiki:
                 gh_command.append("--disable-wiki")
 
-            if not self._issues:
+            if not self.issues:
                 gh_command.append("--disable-issues")
 
             yield run_command(proj_path, gh_command)
 
 
+@dataclass
 class WriteGithubApiToken(AtomicIterable):
-    def __init__(self, repo_name: str):
-        self._repo_name = repo_name
+    repo_name: str
 
     def __call__(
         self, proj_path: ProjectPath, template_dict: Dict, state: Dict, temp_dir: Path
@@ -146,26 +131,26 @@ class WriteGithubApiToken(AtomicIterable):
                     "-b",
                     token,
                     "-r",
-                    self._repo_name,
+                    self.repo_name,
                 ],
             )
 
 
+@dataclass
 class GitignoreWriter(AtomicIterable):
-    def __init__(self, force: bool = False):
-        self._force = force
+    force: bool = False
 
     def __call__(
         self, proj_path: ProjectPath, template_dict: Dict, state: Dict, temp_dir: Path
     ) -> Iterable[RuntimeClosure]:
-        yield JinjaTemplate(JINJA_PATH.gitignore, force=self._force).write(
+        yield JinjaTemplate(JINJA_PATH.gitignore, force=self.force).write(
             proj_path, template_dict, state, proj_path.gitignore
         )
 
 
+@dataclass
 class PrecommitWriter(AtomicIterable):
-    def __init__(self, force: bool = False):
-        self._force = force
+    force: bool = False
 
     def __call__(
         self, proj_path: ProjectPath, template_dict: Dict, state: Dict, temp_dir: Path
@@ -173,13 +158,13 @@ class PrecommitWriter(AtomicIterable):
         yield JinjaTemplate(
             JINJA_PATH.pre_commit,
             executable=True,
-            force=self._force,
+            force=self.force,
         ).write(proj_path, template_dict, state, proj_path.pre_commit)
 
 
+@dataclass
 class GitFileWriter(AtomicIterable):
-    def __init__(self, force: bool = False) -> None:
-        self.force = force
+    force: bool = False
 
     def __call__(
         self, proj_path: ProjectPath, template_dict: Dict, state: Dict, temp_dir: Path
@@ -205,9 +190,9 @@ class GitFileWriter(AtomicIterable):
             yield template.write(proj_path, template_dict, state, target)
 
 
+@dataclass
 class LatexBuildWriter(AtomicIterable):
-    def __init__(self, force: bool = False) -> None:
-        self.force = force
+    force: bool
 
     def __call__(
         self, proj_path: ProjectPath, template_dict: Dict, state: Dict, temp_dir: Path
